@@ -43,7 +43,11 @@
 #include "ext/standard/flock_compat.h"
 #include "ext/standard/md5.h"
 #include "ext/standard/php_var.h"
-#include "apc_arginfo.h"
+#if PHP_VERSION_ID >= 80000
+# include "php_apc_arginfo.h"
+#else
+# include "php_apc_legacy_arginfo.h"
+#endif
 #include "php74_shim.h"
 
 #ifdef HAVE_SYS_FILE_H
@@ -57,21 +61,6 @@
 #include "apc_signal.h"
 #endif
 
-/* {{{ PHP_FUNCTION declarations */
-PHP_FUNCTION(apcu_cache_info);
-PHP_FUNCTION(apcu_clear_cache);
-PHP_FUNCTION(apcu_sma_info);
-PHP_FUNCTION(apcu_key_info);
-PHP_FUNCTION(apcu_store);
-PHP_FUNCTION(apcu_fetch);
-PHP_FUNCTION(apcu_delete);
-PHP_FUNCTION(apcu_add);
-PHP_FUNCTION(apcu_inc);
-PHP_FUNCTION(apcu_dec);
-PHP_FUNCTION(apcu_cas);
-PHP_FUNCTION(apcu_exists);
-/* }}} */
-
 /* {{{ ZEND_DECLARE_MODULE_GLOBALS(apcu) */
 ZEND_DECLARE_MODULE_GLOBALS(apcu)
 
@@ -93,7 +82,7 @@ static void php_apc_init_globals(zend_apcu_globals* apcu_globals)
 	apcu_globals->smart = 0;
 	apcu_globals->preload_path = NULL;
 	apcu_globals->coredump_unmap = 0;
-	apcu_globals->use_request_time = 1;
+	apcu_globals->use_request_time = 0;
 	apcu_globals->serializer_name = NULL;
 	apcu_globals->recursion = 0;
 }
@@ -151,7 +140,7 @@ STD_PHP_INI_BOOLEAN("apc.enable_cli",   "0",    PHP_INI_SYSTEM, OnUpdateBool,
 STD_PHP_INI_BOOLEAN("apc.slam_defense", "0",    PHP_INI_SYSTEM, OnUpdateBool,              slam_defense,     zend_apcu_globals, apcu_globals)
 STD_PHP_INI_ENTRY("apc.preload_path", (char*)NULL,              PHP_INI_SYSTEM, OnUpdateString,       preload_path,  zend_apcu_globals, apcu_globals)
 STD_PHP_INI_BOOLEAN("apc.coredump_unmap", "0", PHP_INI_SYSTEM, OnUpdateBool, coredump_unmap, zend_apcu_globals, apcu_globals)
-STD_PHP_INI_BOOLEAN("apc.use_request_time", "1", PHP_INI_ALL, OnUpdateBool, use_request_time,  zend_apcu_globals, apcu_globals)
+STD_PHP_INI_BOOLEAN("apc.use_request_time", "0", PHP_INI_ALL, OnUpdateBool, use_request_time,  zend_apcu_globals, apcu_globals)
 STD_PHP_INI_ENTRY("apc.serializer", "php", PHP_INI_SYSTEM, OnUpdateStringUnempty, serializer_name, zend_apcu_globals, apcu_globals)
 PHP_INI_END()
 
@@ -461,6 +450,7 @@ static void apc_store_helper(INTERNAL_FUNCTION_PARAMETERS, const zend_bool exclu
 		apc_cache_serializer(apc_user_cache, APCG(serializer_name));
 	}
 
+	/* TODO: Port to array|string for PHP 8? */
 	if (Z_TYPE_P(key) == IS_ARRAY) {
 		zval *hentry;
 		zend_string *hkey;
@@ -475,12 +465,14 @@ static void apc_store_helper(INTERNAL_FUNCTION_PARAMETERS, const zend_bool exclu
 		ZEND_HASH_FOREACH_KEY_VAL(hash, hkey_idx, hkey, hentry) {
 			ZVAL_DEREF(hentry);
 			if (hkey) {
-				if (!apc_cache_store(apc_user_cache, hkey, hentry, (uint32_t) ttl, exclusive)) {
-					zend_hash_add_new(Z_ARRVAL_P(return_value), hkey, &fail_zv);
-				}
+				zend_string_addref(hkey);
 			} else {
-				zend_hash_index_add_new(Z_ARRVAL_P(return_value), hkey_idx, &fail_zv);
+				hkey = zend_long_to_str(hkey_idx);
 			}
+			if (!apc_cache_store(apc_user_cache, hkey, hentry, (uint32_t) ttl, exclusive)) {
+				zend_symtable_add_new(Z_ARRVAL_P(return_value), hkey, &fail_zv);
+			}
+			zend_string_release(hkey);
 		} ZEND_HASH_FOREACH_END();
 		return;
 	} else if (Z_TYPE_P(key) == IS_STRING) {
@@ -500,6 +492,9 @@ static void apc_store_helper(INTERNAL_FUNCTION_PARAMETERS, const zend_bool exclu
 /* {{{ proto bool apcu_enabled(void)
 	returns true when apcu is usable in the current environment */
 PHP_FUNCTION(apcu_enabled) {
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 	RETURN_BOOL(APCG(enabled));
 }
 /* }}} */
@@ -634,6 +629,7 @@ PHP_FUNCTION(apcu_fetch) {
 		convert_to_string(key);
 	}
 
+	/* TODO: Port to array|string for PHP 8? */
 	if (Z_TYPE_P(key) == IS_STRING) {
 		result = apc_cache_fetch(apc_user_cache, Z_STR_P(key), t, return_value);
 	} else if (Z_TYPE_P(key) == IS_ARRAY) {
@@ -684,6 +680,7 @@ PHP_FUNCTION(apcu_exists) {
 		convert_to_string(key);
 	}
 
+	/* TODO: Port to array|string for PHP 8? */
 	if (Z_TYPE_P(key) == IS_STRING) {
 		RETURN_BOOL(apc_cache_exists(apc_user_cache, Z_STR_P(key), t));
 	} else if (Z_TYPE_P(key) == IS_ARRAY) {
@@ -745,13 +742,13 @@ PHP_FUNCTION(apcu_delete) {
 }
 
 PHP_FUNCTION(apcu_entry) {
-	zval *key = NULL;
+	zend_string *key;
 	zend_fcall_info fci = empty_fcall_info;
 	zend_fcall_info_cache fcc = empty_fcall_info_cache;
 	zend_long ttl = 0L;
 	zend_long now = apc_time();
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "zf|l", &key, &fci, &fcc, &ttl) != SUCCESS) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Sf|l", &key, &fci, &fcc, &ttl) != SUCCESS) {
 		return;
 	}
 
@@ -781,35 +778,12 @@ PHP_FUNCTION(apcu_inc_request_time) {
 }
 #endif
 
-/* {{{ apcu_functions[] */
-zend_function_entry apcu_functions[] = {
-	PHP_FE(apcu_cache_info,         arginfo_apcu_cache_info)
-	PHP_FE(apcu_clear_cache,        arginfo_apcu_clear_cache)
-	PHP_FE(apcu_sma_info,           arginfo_apcu_sma_info)
-	PHP_FE(apcu_key_info,           arginfo_apcu_key_info)
-	PHP_FE(apcu_enabled,            arginfo_apcu_enabled)
-	PHP_FE(apcu_store,              arginfo_apcu_store)
-	PHP_FE(apcu_fetch,              arginfo_apcu_fetch)
-	PHP_FE(apcu_delete,             arginfo_apcu_delete)
-	PHP_FE(apcu_add,                arginfo_apcu_store)
-	PHP_FE(apcu_inc,                arginfo_apcu_inc)
-	PHP_FE(apcu_dec,                arginfo_apcu_inc)
-	PHP_FE(apcu_cas,                arginfo_apcu_cas)
-	PHP_FE(apcu_exists,             arginfo_apcu_exists)
-	PHP_FE(apcu_entry,				arginfo_apcu_entry)
-#ifdef APC_DEBUG
-	PHP_FE(apcu_inc_request_time,   NULL)
-#endif
-	PHP_FE_END
-};
-/* }}} */
-
 /* {{{ module definition structure */
 
 zend_module_entry apcu_module_entry = {
 	STANDARD_MODULE_HEADER,
 	PHP_APCU_EXTNAME,
-	apcu_functions,
+	ext_functions,
 	PHP_MINIT(apcu),
 	PHP_MSHUTDOWN(apcu),
 	PHP_RINIT(apcu),
